// sample program to demonstrate bit routines for Clipper
//
// I use these for switches in things like a user profile
// where I can store a lot of flags in a small space.  Like
// security flags.
//
//
//  Another thing I have seen is programmers using an array for tagging
//  records in a tbrowse in a multi-user system.  Using recno() as the
//  offset into the tag array.  This methed limits you to 4096 records
//  in the database.  Using substr(cString,recno(),1) gives you 65,536
//  records max but a bit array will potentially give you room for
//  524,288 records.  Two bit arrays gives you over one million records.
//
//
//
// the functions are:
//
//          bitreset()  <- set all bits off
//              // CATION! space needs to be made for the bit string before
//              //         it is used.
//              cBits := space(4)           // 32 bit fields (8 per character)
//              cBits := bitreset(cBits)
//
//          bitset()    <- set a given bit in a string on or off
//              cBits := bitset(cBits, 28, .t.) // set bit 28 on
//              cBits := bitset(cBits, 14, .f.) // set bit 14 off
//
//          bittest()   <- test if a given bit is set on
//              if (bittest(cBits, 28))         // test if bit 28 is on (.t.)
//                  ? "Bit 28 is on"
//              else
//                  ? "Bit 28 is off"
//              endif
//
//              // toggle a bit between on and off
//              bitset(cBits, 34, ! bittest(cBits, 34))
//

// so output can be redirected  (ie. TEST > save.out )
//
#xtranslate ? => outstd(chr(13)+chr(10))
#xtranslate ? <list,...> => outstd(<list>, chr(13)+chr(10))

procedure main ()
    local   aStruct
    local   i
    local   cBit

    if (! file("TEST.DBF"))
        // there are 8 bits in every character
        // so 10 characters holds 80 bits (thats a lot of switches)
        //
        aStruct := { {"TEST", "C", 10, 0}}
        dbcreate ("TEST", aStruct)
    endif

    use test exclusive new

    if (lastrec() < 1)
        dbappend()
        TEST->test := bitreset(TEST->test)  // set all bits off
    endif

    cBit := TEST->test

    ?
    for i := 1 to (len(cBit) * 8)
        ? bittest(cBit, i)
    next
    ?

    // toggle every other time program is run between all on or every
    // other one on.
    //
    if (bittest(cBit, 1))
        for i := 1 to ((len(cBit) + 6) * 8) step 2
            cBit := bitset(cBit, i, .f.)
        next
    else
        for i := 1 to (len(cBit) * 8) step 2
            cBit := bitset(cBit, i, .t.)
        next
    endif

    ?
    for i := 1 to (len(cBit) * 8)
        ? bittest(cBit, i)
    next
    ?

    // save it back into test.dbf for next try
    TEST->test := cBit

    dbclosearea()

    return
