#include "xfax.ch"
#include "inkey.ch"


PROCEDURE demo( cFileName )

LOCAL nResultCode  := -1
LOCAL nCurrentPage := 0
LOCAL nTotalPage   := 0
LOCAL cTimeStart   := "00:00:00"
LOCAL cTimeElapse  := "00:00:00"
LOCAL lAbort := .F.
LOCAL nCom
LOCAL nFax
LOCAL n

CLEAR SCREEN

// create the serial port object (COM1)
// for COM2 replace the following line with:
//    nCom := xComNew( COM2_BASE, IRQ_3, BAUD_19200 )
//---------------------------------------------------
nCom := xComNew( COM1_BASE, IRQ_4, BAUD_19200 )

IF nCom >= 0
   // create the fax object
   //----------------------
   nFax := xFaxNew( nCom )
   IF nFax >= 0
      IF xFaxErrorNo( nFax ) > 0
         // display error message if there is any error
         //--------------------------------------------
         ? "Error#" + Ltrim(Str(xFaxErrorNo( nFax ))) + ": " + xFaxErrorMsg( nFax )
      ELSE
         // parameter settings
         //-------------------
         xFaxLog( nFax, "fax.log" )
         xFaxLocalID( nFax, "X-Fax Demonstration" )

         IF cFileName == NIL
            // if cFileName is not specified, then start fax reception
            //--------------------------------------------------------
            ? "Waiting for RING ..."

            nResultCode := xFaxBeginReceive( nFax, "", 1 )
         ELSE
            // else send cFileName
            //--------------------
            ? "Sending fax : [" + cFileName + "] ..."

            nResultCode := xFaxBeginSend( nFax, cFileName, "2561 9599" )
         ENDIF

         IF nResultCode == 0
            // start the fax process
            //----------------------
            DO WHILE .T.
               IF LastKey() == K_ESC
                  lAbort := .T.
               ENDIF

               nResultCode = xFaxProcess( nFax, lAbort )

               IF nResultCode == HANDSHAKING
                  @ 0, 67 SAY " Handshaking " COLOR "N/W*"
               ELSEIF nResultCode == CONNECT
                  @ 24, 0
                  ?
                  ? Replicate( "*", 50 )
                  ? "   Local ID: " + xFaxLocalID(  nFax )
                  ? "  Remote ID: " + xFaxRemoteID( nFax )

                  ? "Page Length: "
                  n := xFaxPageLength( nFax )
                  IF n == A4
                     ?? "A4"
                  ELSEIF n == B4
                     ?? "B4"
                  ELSE
                     ?? "Unlimited"
                  ENDIF

                  ? " Resolution: "
                  n := xFaxResolution( nFax )
                  IF n == NORMAL
                     ?? "Normal"
                  ELSE
                     ?? "Fine"
                  ENDIF

                  ? "      Speed: " + LTrim( Str( xFaxSpeed( nFax ), 5 ) )

                  ? "Compression: "
                  n := xFaxCompression( nFax )
                  IF n == MH
                     ?? "MH"
                  ELSE
                     ?? "MR"
                  ENDIF

                  nCurrentPage := xFaxPage( nFax, @nTotalPage )
                  ? " Total Page: " + LTrim(Str(nCurrentPage)) + " / " + LTrim(Str(nTotalPage))
                  ? Replicate( "*", 50 )
                  ?
               ELSEIF nResultCode == TRANSFERING
                  n = 0
                  @ 24, 0 SAY Str( xFaxMeter( nFax, @n ) ) + "/" + Str( n ) +;
                              If( xFaxPause( nFax ), "  < PAUSE >", Space( 11 ) )
               ELSEIF nResultCode != IDLE
                  @ 24, 0

                  IF nResultCode == FINISH
                     ? "End of Process"
                  ELSEIF nResultCode == ABORT
                     ? "Process Aborted"
                  ENDIF

                  EXIT
               ENDIF

               inkey()
            ENDDO
         ENDIF

         ?
         IF nResultCode == FINISH
            ?
         ELSEIF nResultCode != 0
            ? "Error#" + Ltrim(Str(xFaxErrorNo( nFax ))) + ": "
         ENDIF
         ?? xFaxErrorMsg( nFax )

         n = xFaxDuration( nFax, @cTimeStart, @cTimeElapse )
         ?
         ? "Time Start : " + cTimeStart
         ? "Time Elapse: " + cTimeElapse + "  (" + Ltrim( Str( n ) ) + " seconds)"
      ENDIF

      // delete the fax object
      //----------------------
      xFaxDelete( nFax )
   ENDIF

   // delete the serial port object
   //------------------------------
   xComDelete( nCom )
ENDIF

RETURN
